import { EventClient } from '../event/event-client';
import { DispatcherClient } from '../dispatcher/dispatcher-client';
import { AdminClient } from '../admin/admin-client';
import { CallOptions, ChannelCredentials, ClientMiddlewareCall } from 'nice-grpc';
import { Workflow } from '../../workflow';
import { Worker, WorkerOpts } from '../worker';
import Logger from '../../util/logger/logger';
import { AxiosRequestConfig } from 'axios';
import { ClientConfig } from './client-config';
import { ListenerClient } from '../listener/listener-client';
import { Api } from '../rest/generated/Api';
export interface HatchetClientOptions {
    config_path?: string;
    credentials?: ChannelCredentials;
}
export declare const channelFactory: (config: ClientConfig, credentials: ChannelCredentials) => import("nice-grpc").Channel;
export declare const addTokenMiddleware: (token: string) => <Request, Response>(call: ClientMiddlewareCall<Request, Response>, options: CallOptions) => AsyncGenerator<Awaited<Response>, Awaited<Response> | undefined, undefined>;
export declare class HatchetClient {
    config: ClientConfig;
    credentials: ChannelCredentials;
    event: EventClient;
    dispatcher: DispatcherClient;
    admin: AdminClient;
    api: Api;
    listener: ListenerClient;
    tenantId: string;
    logger: Logger;
    constructor(config?: Partial<ClientConfig>, options?: HatchetClientOptions, axiosOpts?: AxiosRequestConfig);
    static init(config?: Partial<ClientConfig>, options?: HatchetClientOptions, axiosConfig?: AxiosRequestConfig): HatchetClient;
    run(workflow: string | Workflow): Promise<Worker>;
    worker(workflow: string | Workflow, opts?: Omit<WorkerOpts, 'name'> | number): Promise<Worker>;
    webhooks(workflows: Workflow[]): import("../worker/handler").WebhookHandler;
}
