"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sdk_1 = __importDefault(require("../sdk"));
const hatchet = sdk_1.default.init();
const parentWorkflow = {
    id: 'parent-workflow',
    description: 'simple example for spawning child workflows',
    on: {
        event: 'fanout:create',
    },
    steps: [
        {
            name: 'parent-spawn',
            timeout: '70s',
            run: (ctx) => __awaiter(void 0, void 0, void 0, function* () {
                const promises = Array.from({ length: 3 }, (_, i) => ctx
                    .spawnWorkflow('child-workflow', { input: `child-input-${i}` }, { additionalMetadata: { childKey: 'childValue' } })
                    .result()
                    .then((result) => {
                    ctx.log('spawned workflow result:');
                    return result;
                }));
                const results = yield Promise.all(promises);
                console.log('spawned workflow results:', results);
                console.log('number of spawned workflows:', results.length);
                return { spawned: 'x' };
            }),
        },
    ],
};
const childWorkflow = {
    id: 'child-workflow',
    description: 'simple example for spawning child workflows',
    on: {
        event: 'child:create',
    },
    steps: [
        {
            name: 'child-work',
            run: (ctx) => __awaiter(void 0, void 0, void 0, function* () {
                const { input } = ctx.workflowInput();
                // throw new Error('child error');
                return { 'child-output': 'sm' };
            }),
        },
        {
            name: 'child-work2',
            run: (ctx) => __awaiter(void 0, void 0, void 0, function* () {
                const { input } = ctx.workflowInput();
                // Perform CPU-bound work
                // throw new Error('child error');
                console.log('child workflow input:', input);
                // Generate a large amount of garbage data
                const garbageData = Array.from({ length: 1e6 / 3.5 }, (_, i) => `garbage-${i}`).join(',');
                console.log('Generated garbage data:', `${garbageData.slice(0, 100)}...`); // Print a snippet of the garbage data
                return { 'child-output': garbageData };
            }),
        },
        {
            name: 'child-work3',
            parents: ['child-work'],
            run: (ctx) => __awaiter(void 0, void 0, void 0, function* () {
                const { input } = ctx.workflowInput();
                // throw new Error('child error');
                const garbageData = Array.from({ length: 1e6 / 3.5 }, (_, i) => `garbage-${i}`).join(',');
                console.log('Generated garbage data:', `${garbageData.slice(0, 100)}...`); // Print a snippet of the garbage data
                return { 'child-output': garbageData };
            }),
        },
    ],
};
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        const worker = yield hatchet.worker('fanout-worker', { maxRuns: 1000 });
        yield worker.registerWorkflow(parentWorkflow);
        yield worker.registerWorkflow(childWorkflow);
        worker.start();
    });
}
main();
