"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Context = exports.ContextWorker = exports.CreateStepSchema = exports.DesiredWorkerLabelSchema = exports.CreateRateLimitSchema = void 0;
exports.mapRateLimit = mapRateLimit;
/* eslint-disable max-classes-per-file */
const hatchet_error_1 = __importDefault(require("./util/errors/hatchet-error"));
const z = __importStar(require("zod"));
const workflow_1 = require("./workflow");
const logger_1 = require("./util/logger");
const parse_1 = require("./util/parse");
const workflows_1 = require("./protoc/workflows");
exports.CreateRateLimitSchema = z.object({
    key: z.string().optional(),
    staticKey: z.string().optional(),
    dynamicKey: z.string().optional(),
    units: z.union([z.number().min(0), z.string()]),
    limit: z.union([z.number().min(1), z.string()]).optional(),
    duration: z.nativeEnum(workflows_1.RateLimitDuration).optional(),
});
exports.DesiredWorkerLabelSchema = z
    .union([
    z.string(),
    z.number().int(),
    z.object({
        value: z.union([z.string(), z.number()]),
        required: z.boolean().optional(),
        weight: z.number().int().optional(),
        // (optional) comparator for the label
        // if not provided, the default is EQUAL
        // desired COMPARATOR actual (i.e. desired > actual for GREATER_THAN)
        comparator: z.nativeEnum(workflows_1.WorkerLabelComparator).optional(),
    }),
])
    .optional();
exports.CreateStepSchema = z.object({
    name: z.string(),
    parents: z.array(z.string()).optional(),
    timeout: workflow_1.HatchetTimeoutSchema.optional(),
    retries: z.number().optional(),
    rate_limits: z.array(exports.CreateRateLimitSchema).optional(),
    worker_labels: z.record(z.lazy(() => exports.DesiredWorkerLabelSchema)).optional(),
});
class ContextWorker {
    constructor(worker) {
        this.worker = worker;
    }
    id() {
        return this.worker.workerId;
    }
    hasWorkflow(workflowName) {
        return !!this.worker.workflow_registry.find((workflow) => workflow.id === workflowName);
    }
    labels() {
        return this.worker.labels;
    }
    upsertLabels(labels) {
        return this.worker.upsertLabels(labels);
    }
}
exports.ContextWorker = ContextWorker;
class Context {
    constructor(action, client, worker) {
        this.controller = new AbortController();
        this.overridesData = {};
        this.spawnIndex = 0;
        try {
            const data = (0, parse_1.parseJSON)(action.actionPayload);
            this.data = data;
            this.action = action;
            this.client = client;
            this.worker = new ContextWorker(worker);
            this.logger = new logger_1.Logger(`Context Logger`, client.config.log_level);
            // if this is a getGroupKeyRunId, the data is the workflow input
            if (action.getGroupKeyRunId !== '') {
                this.input = data;
            }
            else {
                this.input = data.input;
            }
            this.overridesData = data.overrides || {};
        }
        catch (e) {
            throw new hatchet_error_1.default(`Could not parse payload: ${e.message}`);
        }
    }
    stepOutput(step) {
        if (!this.data.parents) {
            throw new hatchet_error_1.default('Step output not found');
        }
        if (!this.data.parents[step]) {
            throw new hatchet_error_1.default(`Step output for '${step}' not found`);
        }
        return this.data.parents[step];
    }
    triggeredByEvent() {
        var _a;
        return ((_a = this.data) === null || _a === void 0 ? void 0 : _a.triggered_by) === 'event';
    }
    workflowInput() {
        return this.input;
    }
    workflowName() {
        return this.action.jobName;
    }
    userData() {
        var _a;
        return (_a = this.data) === null || _a === void 0 ? void 0 : _a.user_data;
    }
    stepName() {
        return this.action.stepName;
    }
    workflowRunId() {
        return this.action.workflowRunId;
    }
    retryCount() {
        return this.action.retryCount;
    }
    playground(name, defaultValue = '') {
        if (name in this.overridesData) {
            return this.overridesData[name];
        }
        this.client.dispatcher.putOverridesData({
            stepRunId: this.action.stepRunId,
            path: name,
            value: JSON.stringify(defaultValue),
        });
        return defaultValue;
    }
    log(message, level) {
        const { stepRunId } = this.action;
        if (!stepRunId) {
            // log a warning
            this.logger.warn('cannot log from context without stepRunId');
            return;
        }
        this.client.event.putLog(stepRunId, message, level);
    }
    /**
     * Refreshes the timeout for the current step.
     * @param incrementBy - The interval by which to increment the timeout.
     *                     The interval should be specified in the format of '10s' for 10 seconds,
     *                     '1m' for 1 minute, or '1d' for 1 day.
     */
    refreshTimeout(incrementBy) {
        return __awaiter(this, void 0, void 0, function* () {
            const { stepRunId } = this.action;
            if (!stepRunId) {
                // log a warning
                this.logger.warn('cannot refresh timeout from context without stepRunId');
                return;
            }
            yield this.client.dispatcher.refreshTimeout(incrementBy, stepRunId);
        });
    }
    releaseSlot() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.dispatcher.client.releaseSlot({
                stepRunId: this.action.stepRunId,
            });
        });
    }
    putStream(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const { stepRunId } = this.action;
            if (!stepRunId) {
                // log a warning
                this.logger.warn('cannot log from context without stepRunId');
                return;
            }
            yield this.client.event.putStream(stepRunId, data);
        });
    }
    /**
     * Spawns multiple workflows.
     *
     * @param workflows an array of objects containing the workflow name, input data, and options for each workflow
     * @returns a list of references to the spawned workflow runs
     */
    spawnWorkflows(workflows) {
        const { workflowRunId, stepRunId } = this.action;
        const workflowRuns = workflows.map(({ workflow, input, options }) => {
            let workflowName;
            if (typeof workflow === 'string') {
                workflowName = workflow;
            }
            else {
                workflowName = workflow.id;
            }
            const name = this.client.config.namespace + workflowName;
            let key;
            let sticky = false;
            let metadata;
            if (options) {
                key = options.key;
                sticky = options.sticky;
                metadata = options.additionalMetadata;
            }
            if (sticky && !this.worker.hasWorkflow(name)) {
                throw new hatchet_error_1.default(`Cannot run with sticky: workflow ${name} is not registered on the worker`);
            }
            const resp = {
                workflowName: name,
                input,
                options: {
                    parentId: workflowRunId,
                    parentStepRunId: stepRunId,
                    childKey: key,
                    childIndex: this.spawnIndex,
                    desiredWorkerId: sticky ? this.worker.id() : undefined,
                    additionalMetadata: metadata,
                },
            };
            this.spawnIndex += 1;
            return resp;
        });
        try {
            const resp = this.client.admin.runWorkflows(workflowRuns);
            return resp;
        }
        catch (e) {
            throw new hatchet_error_1.default(e.message);
        }
    }
    /**
     * Spawns a new workflow.
     *
     * @param workflowName the name of the workflow to spawn
     * @param input the input data for the workflow
     * @param options additional options for spawning the workflow. If a string is provided, it is used as the key.
     *                If an object is provided, it can include:
     *                - key: a unique identifier for the workflow (deprecated, use options.key instead)
     *                - sticky: a boolean indicating whether to use sticky execution
     * @param <Q> the type of the input data
     * @param <P> the type of the output data
     * @return a reference to the spawned workflow run
     */
    spawnWorkflow(workflow, input, options) {
        const { workflowRunId, stepRunId } = this.action;
        let workflowName = '';
        if (typeof workflow === 'string') {
            workflowName = workflow;
        }
        else {
            workflowName = workflow.id;
        }
        const name = this.client.config.namespace + workflowName;
        let key = '';
        let sticky = false;
        let metadata;
        if (typeof options === 'string') {
            this.logger.warn('Using key param is deprecated and will be removed in a future release. Use options.key instead.');
            key = options;
        }
        else {
            key = options === null || options === void 0 ? void 0 : options.key;
            sticky = options === null || options === void 0 ? void 0 : options.sticky;
            metadata = options === null || options === void 0 ? void 0 : options.additionalMetadata;
        }
        if (sticky && !this.worker.hasWorkflow(name)) {
            throw new hatchet_error_1.default(`cannot run with sticky: workflow ${name} is not registered on the worker`);
        }
        try {
            const resp = this.client.admin.runWorkflow(name, input, {
                parentId: workflowRunId,
                parentStepRunId: stepRunId,
                childKey: key,
                childIndex: this.spawnIndex,
                desiredWorkerId: sticky ? this.worker.id() : undefined,
                additionalMetadata: metadata,
            });
            this.spawnIndex += 1;
            return resp;
        }
        catch (e) {
            throw new hatchet_error_1.default(e.message);
        }
    }
    additionalMetadata() {
        if (!this.action.additionalMetadata) {
            return {};
        }
        // parse the additional metadata
        const res = (0, parse_1.parseJSON)(this.action.additionalMetadata);
        return res;
    }
    childIndex() {
        return this.action.childWorkflowIndex;
    }
    childKey() {
        return this.action.childWorkflowKey;
    }
    parentWorkflowRunId() {
        return this.action.parentWorkflowRunId;
    }
}
exports.Context = Context;
function mapRateLimit(limits) {
    if (!limits)
        return [];
    return limits.map((l) => {
        let key = l.staticKey;
        const keyExpression = l.dynamicKey;
        if (l.key !== undefined) {
            // eslint-disable-next-line no-console
            console.warn('key is deprecated and will be removed in a future release, please use staticKey instead');
            key = l.key;
        }
        if (keyExpression !== undefined) {
            if (key !== undefined) {
                throw new Error('Cannot have both static key and dynamic key set');
            }
            key = keyExpression;
            if (!validateCelExpression(keyExpression)) {
                throw new Error(`Invalid CEL expression: ${keyExpression}`);
            }
        }
        if (key === undefined) {
            throw new Error(`Invalid key`);
        }
        let units;
        let unitsExpression;
        if (typeof l.units === 'number') {
            units = l.units;
        }
        else {
            if (!validateCelExpression(l.units)) {
                throw new Error(`Invalid CEL expression: ${l.units}`);
            }
            unitsExpression = l.units;
        }
        let limitExpression;
        if (l.limit !== undefined) {
            if (typeof l.limit === 'number') {
                limitExpression = `${l.limit}`;
            }
            else {
                if (!validateCelExpression(l.limit)) {
                    throw new Error(`Invalid CEL expression: ${l.limit}`);
                }
                limitExpression = l.limit;
            }
        }
        if (keyExpression !== undefined && limitExpression === undefined) {
            throw new Error('CEL based keys requires limit to be set');
        }
        return {
            key,
            keyExpr: keyExpression,
            units,
            unitsExpr: unitsExpression,
            limitValuesExpr: limitExpression,
            duration: l.duration,
        };
    });
}
// Helper function to validate CEL expressions
function validateCelExpression(expr) {
    // This is a placeholder. In a real implementation, you'd need to use a CEL parser or validator.
    // For now, we'll just return true to mimic the behavior.
    return true;
}
